@@ -4,15 +4,13 @@ use Parse::Stallion;
 
 my %calculator_rules = (
  start_expression => {
-   rule_type => 'and',
-   composed_of => ['expression', 'end_of_string'],
+   and => ['expression', 'end_of_string'],
    evaluation => sub {return $_[0]->{expression}},
   }
 ,
  expression => {
-   rule_type => 'and',
-   composed_of => ['term', 
-    {repeating => {composed_of => ['plus_or_minus', 'term'],},},],
+   and => ['term', 
+    {multiple => {and => ['plus_or_minus', 'term'],},},],
    evaluation => sub {my $to_combine = $_[0]->{term};
     my $plus_or_minus = $_[0]->{plus_or_minus};
     my $value = shift @$to_combine;
@@ -29,8 +27,8 @@ my %calculator_rules = (
   },
 ,
  term => {
-   composed_of => ['factor', 
-    {repeating => {composed_of => ['times_or_divide_or_modulo', 'factor'],},},],
+   and => ['factor', 
+    {multiple => {and => ['times_or_divide_or_modulo', 'factor'],},},],
    evaluation => sub {my $to_combine = $_[0]->{factor};
     my $times_or_divide_or_modulo = $_[0]->{times_or_divide_or_modulo};
     my $value = shift @$to_combine;
@@ -51,8 +49,8 @@ my %calculator_rules = (
   },
 ,
  factor => {
-   composed_of => ['fin_exp', 
-    {repeating => {composed_of => ['power_of', 'fin_exp'],},},],
+   and => ['fin_exp', 
+    {multiple => {and => ['power_of', 'fin_exp'],},},],
    evaluation => sub {my $to_combine = $_[0]->{fin_exp};
     my $value = pop @$to_combine;
     while ($#{$to_combine} > -1) {
@@ -63,13 +61,12 @@ my %calculator_rules = (
   },
 ,
 fin_exp => {
-  rule_type => 'or',
-  any_one_of => [
-    {composed_of => ['left_parenthesis', 'expression', 'right_parenthesis'],
+  or => [
+    {and => ['left_parenthesis', 'expression', 'right_parenthesis'],
      evaluation => sub {return $_[0]->{expression} },
      precedence => 0,
     },
-    {composed_of => ['number'],
+    {and => ['number'],
      evaluation => sub {return $_[0]->{number} },
      precedence => 0,
     },
@@ -77,12 +74,10 @@ fin_exp => {
   },
 ,
 end_of_string => {
-  rule_type => 'leaf',
   regex_match => qr/\z/,
  },
 ,
 number => {
-  rule_type => 'leaf',
   regex_match => qr/\s*[+-]?(\d+(\.\d*)?|\.\d+)\s*/,
   evaluation => sub{
    return 0 + $_[0];
@@ -90,91 +85,64 @@ number => {
  },
 ,
 left_parenthesis => {
-  rule_type => 'leaf',
   regex_match => qr/\s*\(\s*/,
  },
 ,
 right_parenthesis => {
-  rule_type => 'leaf',
   regex_match => qr/\s*\)\s*/,
  },
 ,
 power_of => {
-  rule_type => 'leaf',
   regex_match => qr/\s*\*\*\s*/,
  },
 ,
 plus_or_minus => {
-  rule_type => 'or',
-  any_one_of => ['plus', 'minus'],
+  or => ['plus', 'minus'],
  },
 ,
 plus => {
-  rule_type => 'leaf',
   regex_match => qr/\s*\+\s*/,
  },
 ,
 minus => {
-  rule_type => 'leaf',
   regex_match => qr/\s*\-\s*/,
  },
 ,
 times_or_divide_or_modulo => {
-  rule_type => 'or',
-  any_one_of => ['times', 'divided_by', 'modulo'],
+  or => ['times', 'divided_by', 'modulo'],
  },
 ,
 modulo => {
-  rule_type => 'leaf',
   regex_match => qr/\s*\%\s*/,
  },
 ,
 times => {
-  rule_type => 'leaf',
   regex_match => qr/\s*\*\s*/,
  },
 ,
 divided_by => {
-  rule_type => 'leaf',
   regex_match => qr/\s*\/\s*/,
  },
 ,
 );
 
-my $calculator_parser = new Parse::Stallion();
-foreach my $rule_name (keys %calculator_rules) {
-  $calculator_parser->add_rule({rule_name => $rule_name,
- %{$calculator_rules{$rule_name}}});
-}
+my $calculator_parser = new Parse::Stallion({
+  start_rule => 'start_expression',
+  rules_to_set_up_hash => \%calculator_rules});
 $calculator_parser->generate_evaluate_subroutines;
 
-my $result =
- $calculator_parser->parse({initial_node => 'start_expression',
-   parse_this=>"7+4"});
-my $parsed_tree = $result->{tree};
-$result = $calculator_parser->do_tree_evaluation({tree=>$parsed_tree});
+my $result = $calculator_parser->parse_and_evaluate("7+4");
 print "should be 11, result is $result\n";
 
-$result =
- $calculator_parser->parse({initial_node => 'start_expression',
- parse_this=>"7*4"});
-$parsed_tree = $result->{tree};
-$result = $calculator_parser->do_tree_evaluation({tree=>$parsed_tree});
+$result = $calculator_parser->parse_and_evaluate("7*4");
 print "should be 28, Result is $result\n";
 
-$result =
- $calculator_parser->parse({initial_node => 'start_expression',
- parse_this=>"3+7*4"});
-$parsed_tree = $result->{tree};
-$result = $calculator_parser->do_tree_evaluation({tree=>$parsed_tree});
+$result = $calculator_parser->parse_and_evaluate("3+7*4");
 print "should be 31, result is $result\n";
 
-$result =
- eval {
- $calculator_parser->parse_and_evaluate({initial_node => 'start_expression',
- parse_this=>"3+-+7*4"})};
+($result, $parse_info) = $calculator_parser->parse_and_evaluate("3+-+7*4");
 
-print "should be 1, Parse failed: ".$calculator_parser->parse_failed."\n";
+print "should be 1, Parse failed: ".$parse_info->{parse_failed}."\n";
 
 print "\nAll done\n";
 
